@@ -21,16 +21,13 @@ private Q_SLOTS:
     void validStructures();
     void validPackages();
     void wallpaperPackage();
-    void copyPerformance();
     void mutateAfterCopy();
     void emptyContentsPrefix();
-    void multiplePaths();
     void directories();
     void requiredDirectories();
     void files();
     void requiredFiles();
     void path();
-    void name();
     void required();
     void mimeTypes();
     void customContent();
