@@ -21,13 +21,13 @@
 #include <QCommandLineParser>
 #include <QDir>
 #include <QFileInfo>
+#include <QList>
 #include <QMap>
 #include <QRegularExpression>
 #include <QStandardPaths>
 #include <QStringList>
 #include <QTimer>
 #include <QUrl>
-#include <QVector>
 #include <QXmlStreamWriter>
 
 #include <iomanip>
@@ -36,9 +36,6 @@
 #include "options.h"
 
 #include "../kpackage/config-package.h"
-// for the index creation function
-#include "../kpackage/package_export.h"
-#include "../kpackage/private/packagejobthread_p.h"
 
 #include "kpackage_debug.h"
 
@@ -53,13 +50,12 @@ public:
     QString packageRoot;
     QString packageFile;
     QString package;
-    QStringList pluginTypes;
-    KPackage::Package installer;
+    QString kpackageType = QStringLiteral("KPackage/Generic");
     KPluginMetaData metadata;
     QString installPath;
     void output(const QString &msg);
-    QStringList packages(const QStringList &types, const QString &path = QString());
-    void renderTypeTable(const QMap<QString, QStringList> &plugins);
+    QStringList packages(const QString &type, const QString &path = QString());
+    void renderTypeTable(const QMap<QString, QString> &plugins);
     void listTypes();
     void coutput(const QString &msg);
     void cerror(const QString &msg);
@@ -81,9 +77,9 @@ PackageTool::~PackageTool()
 
 void PackageTool::runMain()
 {
-    KPackage::PackageStructure structure;
     if (d->parser->isSet(Options::hash())) {
         const QString path = d->parser->value(Options::hash());
+        KPackage::PackageStructure structure;
         KPackage::Package package(&structure);
         package.setPath(path);
         const QString hash = QString::fromLocal8Bit(package.cryptographicHash(QCryptographicHash::Sha1));
@@ -103,9 +99,10 @@ void PackageTool::runMain()
         return;
     }
 
-    QString type = d->parser->value(Options::type());
-    d->pluginTypes.clear();
-    d->installer = Package();
+    if (d->parser->isSet(Options::type())) {
+        d->kpackageType = d->parser->value(Options::type());
+    }
+    d->packageRoot = KPackage::PackageLoader::self()->loadPackage(d->kpackageType).defaultPackageRoot();
 
     if (d->parser->isSet(Options::remove())) {
         d->package = d->parser->value(Options::remove());
@@ -129,46 +126,10 @@ void PackageTool::runMain()
         d->packageFile = d->package;
     }
 
-    if (!d->packageFile.isEmpty()
-        && (!d->parser->isSet(Options::type()) || type.compare(i18nc("package type", "wallpaper"), Qt::CaseInsensitive) == 0
-            || type.compare(QLatin1String("wallpaper"), Qt::CaseInsensitive) == 0)) {
-        // Check type for common plasma packages
-        KPackage::Package package(&structure);
-        QString serviceType;
-        package.setPath(d->packageFile);
-
-        if (package.isValid() && package.metadata().isValid()) {
-            serviceType = package.metadata().value(QStringLiteral("X-Plasma-ServiceType"));
-            const auto serviceTypes = readKPackageTypes(package.metadata());
-            if (serviceType.isEmpty() && !serviceTypes.isEmpty()) {
-                serviceType = serviceTypes.first();
-            }
-        }
-
-        if (!serviceType.isEmpty()) {
-            if (serviceType == QLatin1String("KPackage/Generic")) {
-                type = QStringLiteral("KPackage/Generic");
-            } else {
-                type = serviceType;
-                // qDebug() << "fallthrough type is" << serviceType;
-            }
-        }
+    if (!PackageLoader::self()->loadPackageStructure(d->kpackageType)) {
+        qWarning() << "Package type" << d->kpackageType << "not found";
     }
 
-    {
-        PackageStructure *structure = PackageLoader::self()->loadPackageStructure(type);
-
-        if (structure) {
-            d->installer = Package(structure);
-        }
-
-        if (!d->installer.hasValidStructure()) {
-            qWarning() << "Package type" << type << "not found";
-        }
-
-        d->packageRoot = d->installer.defaultPackageRoot();
-        d->pluginTypes << type;
-    }
     if (d->parser->isSet(Options::show())) {
         const QString pluginName = d->package;
         showPackageInfo(pluginName);
@@ -180,96 +141,58 @@ void PackageTool::runMain()
     }
 
     if (d->parser->isSet(Options::list())) {
-        d->packageRoot = findPackageRoot(d->package, d->packageRoot);
-        d->coutput(i18n("Listing service types: %1 in %2", d->pluginTypes.join(QStringLiteral(", ")), d->packageRoot));
-        listPackages(d->pluginTypes, d->packageRoot);
+        QString packageRoot = resolvePackageRootWithOptions();
+        d->coutput(i18n("Listing KPackageType: %1 in %2", d->kpackageType, packageRoot));
+        listPackages(d->kpackageType, packageRoot);
         exit(0);
-
-    } else if (d->parser->isSet(Options::generateIndex())) {
-        // TODO KF6 Remove
-        qWarning() << "The indexing feature is removed in KPackage 5.82";
-        exit(0);
-
-    } else if (d->parser->isSet(Options::removeIndex())) {
-        // TODO KF6 Remove
-        qWarning() << "The indexing feature is removed in KPackage 5.82";
-        exit(0);
-
     } else {
         // install, remove or upgrade
-        if (!d->installer.isValid()) {
-            d->installer = KPackage::Package(new KPackage::PackageStructure());
-        }
-
-        d->packageRoot = findPackageRoot(d->package, d->packageRoot);
+        d->packageRoot = resolvePackageRootWithOptions();
 
         if (d->parser->isSet(Options::remove()) || d->parser->isSet(Options::upgrade())) {
             QString pkgPath;
-            for (const QString &t : std::as_const(d->pluginTypes)) {
-                KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage(t);
-                pkg.setPath(d->package);
-                if (pkg.isValid()) {
-                    pkgPath = pkg.path();
-                    if (pkgPath.isEmpty() && !d->packageFile.isEmpty()) {
-                        pkgPath = d->packageFile;
-                    }
-                    continue;
+            KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage(d->kpackageType);
+            pkg.setPath(d->package);
+            if (pkg.isValid()) {
+                pkgPath = pkg.path();
+                if (pkgPath.isEmpty() && !d->packageFile.isEmpty()) {
+                    pkgPath = d->packageFile;
                 }
             }
             if (pkgPath.isEmpty()) {
                 pkgPath = d->package;
             }
-
-            if (d->parser->isSet(Options::upgrade())) {
-                d->installer.setPath(d->package);
-            }
             QString _p = d->packageRoot;
             if (!_p.endsWith(QLatin1Char('/'))) {
                 _p.append(QLatin1Char('/'));
             }
             _p.append(d->package);
-            d->installer.setDefaultPackageRoot(d->packageRoot);
-            d->installer.setPath(pkgPath);
 
             if (!d->parser->isSet(Options::type())) {
-                const QStringList lst = readKPackageTypes(d->installer.metadata());
-                for (const QString &st : lst) {
-                    if (!d->pluginTypes.contains(st)) {
-                        d->pluginTypes << st;
-                    }
-                }
+                d->kpackageType = readKPackageType(pkg.metadata());
             }
 
             QString pluginName;
-            if (d->installer.isValid()) {
-                d->metadata = d->installer.metadata();
+            if (pkg.metadata().isValid()) {
+                d->metadata = pkg.metadata();
                 if (!d->metadata.isValid()) {
                     pluginName = d->package;
                 } else if (!d->metadata.isValid() && d->metadata.pluginId().isEmpty()) {
-                    // plugin name given in command line
+                    // plugin id given in command line
                     pluginName = d->package;
                 } else {
-                    // Parameter was a plasma package, get plugin name from the package
+                    // Parameter was a plasma package, get plugin id from the package
                     pluginName = d->metadata.pluginId();
                 }
             }
-            QStringList installed = d->packages(d->pluginTypes);
+            QStringList installed = d->packages(d->kpackageType);
 
-            if (QFile::exists(d->packageFile)) {
-                d->installer.setPath(d->packageFile);
-                if (d->installer.isValid()) {
-                    if (d->installer.metadata().isValid()) {
-                        pluginName = d->installer.metadata().pluginId();
-                    }
-                }
-            }
             // Uninstalling ...
-            if (installed.contains(pluginName)) { // Assume it's a plugin name
-                d->installer.setPath(pluginName);
-                KJob *uninstallJob = d->installer.uninstall(pluginName, d->packageRoot);
-                // clang-format off
-                connect(uninstallJob, SIGNAL(result(KJob*)), SLOT(packageUninstalled(KJob*)));
-                // clang-format on
+            if (installed.contains(pluginName)) { // Assume it's a plugin id
+                KPackage::PackageJob *uninstallJob = KPackage::PackageJob::uninstall(d->kpackageType, pluginName, d->packageRoot);
+                connect(uninstallJob, &KPackage::PackageJob::finished, this, [uninstallJob, this]() {
+                    packageUninstalled(uninstallJob);
+                });
                 return;
             } else {
                 d->coutput(i18n("Error: Plugin %1 is not installed.", pluginName));
@@ -277,10 +200,10 @@ void PackageTool::runMain()
             }
         }
         if (d->parser->isSet(Options::install())) {
-            KJob *installJob = d->installer.install(d->packageFile, d->packageRoot);
-            // clang-format off
-            connect(installJob, SIGNAL(result(KJob*)), SLOT(packageInstalled(KJob*)));
-            // clang-format on
+            auto installJob = KPackage::PackageJob::install(d->kpackageType, d->packageFile, d->packageRoot);
+            connect(installJob, &KPackage::PackageJob::finished, this, [installJob, this]() {
+                packageInstalled(installJob);
+            });
             return;
         }
         if (d->package.isEmpty()) {
@@ -304,31 +227,21 @@ void PackageToolPrivate::cerror(const QString &msg)
     (*cerr).flush();
 }
 
-QStringList PackageToolPrivate::packages(const QStringList &types, const QString &path)
+QStringList PackageToolPrivate::packages(const QString &type, const QString &path)
 {
     QStringList result;
-
-    for (const QString &type : types) {
-        const QList<KPluginMetaData> services = KPackage::PackageLoader::self()->listPackages(type, path);
-        for (const KPluginMetaData &service : services) {
-            const QString _plugin = service.pluginId();
-            if (!result.contains(_plugin)) {
-                result << _plugin;
-            }
+    const QList<KPluginMetaData> dataList = KPackage::PackageLoader::self()->listPackages(type, path);
+    for (const KPluginMetaData &data : dataList) {
+        if (!result.contains(data.pluginId())) {
+            result << data.pluginId();
         }
     }
-
     return result;
 }
 
 void PackageTool::showPackageInfo(const QString &pluginName)
 {
-    QString type = QStringLiteral("KPackage/Generic");
-    if (!d->pluginTypes.contains(type) && !d->pluginTypes.isEmpty()) {
-        type = d->pluginTypes.at(0);
-    }
-    KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage(type);
-
+    KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage(d->kpackageType);
     pkg.setDefaultPackageRoot(d->packageRoot);
 
     if (QFile::exists(d->packageFile)) {
@@ -344,12 +257,16 @@ void PackageTool::showPackageInfo(const QString &pluginName)
         return;
     }
     d->coutput(i18n("Showing info for package: %1", pluginName));
-    d->coutput(i18n("      Name : %1", i.name()));
-    d->coutput(i18n("   Comment : %1", i.value(QStringLiteral("Comment"))));
-    d->coutput(i18n("    Plugin : %1", i.pluginId()));
+    d->coutput(i18n("  Name       : %1", i.name()));
+    d->coutput(i18n("  Description: %1", i.description()));
+    d->coutput(i18n("  Plugin     : %1", i.pluginId()));
     auto const authors = i.authors();
-    d->coutput(i18n("    Author : %1", authors.first().name()));
-    d->coutput(i18n("      Path : %1", pkg.path()));
+    QStringList authorNames;
+    for (const KAboutPerson &author : authors) {
+        authorNames << author.name();
+    }
+    d->coutput(i18n("  Author     : %1", authorNames.join(QLatin1String(", "))));
+    d->coutput(i18n("  Path       : %1", pkg.path()));
 
     exit(0);
 }
@@ -396,16 +313,10 @@ void PackageTool::showAppstreamInfo(const QString &pluginName)
     // This can happen in the case an application wanting to support kpackage based extensions includes in the same project both the packagestructure plugin and
     // the packages themselves. In that case at build time the packagestructure plugin wouldn't be installed yet
 
-    if (QFile::exists(pluginName + QStringLiteral("/metadata.desktop"))) {
-        i = KPluginMetaData::fromDesktopFile(pluginName + QStringLiteral("/metadata.desktop"), {QStringLiteral(":/kservicetypes5/kpackage-generic.desktop")});
-    } else if (QFile::exists(pluginName + QStringLiteral("/metadata.json"))) {
+    if (QFile::exists(pluginName + QStringLiteral("/metadata.json"))) {
         i = KPluginMetaData::fromJsonFile(pluginName + QStringLiteral("/metadata.json"));
     } else {
-        QString type = QStringLiteral("KPackage/Generic");
-        if (!d->pluginTypes.contains(type) && !d->pluginTypes.isEmpty()) {
-            type = d->pluginTypes.at(0);
-        }
-        KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage(type);
+        KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage(d->kpackageType);
 
         pkg.setDefaultPackageRoot(d->packageRoot);
 
@@ -425,23 +336,6 @@ void PackageTool::showAppstreamInfo(const QString &pluginName)
     }
     QString parentApp = i.value(QLatin1String("X-KDE-ParentApp"));
 
-#if KPACKAGE_BUILD_DEPRECATED_SINCE(5, 85)
-    KPluginMetaData packageStructureMetaData;
-    {
-        const QStringList packageFormats = readKPackageTypes(i);
-        if (!packageFormats.isEmpty()) {
-            packageStructureMetaData = structureForKPackageType(packageFormats.first());
-        }
-    }
-    if (parentApp.isEmpty()) {
-        parentApp = packageStructureMetaData.value(QLatin1String("X-KDE-ParentApp"));
-        if (!parentApp.isEmpty()) {
-            qCDebug(KPACKAGE_LOG) << "Implicitly specifying X-KDE-ParentApp by it's parent structure is deprecated and will"
-                                     "be removed in KF6. Either the value should be explicitly set or the default will be used";
-        }
-    }
-#endif
-
     if (i.value(QStringLiteral("NoDisplay"), false)) {
         std::exit(0);
     }
@@ -455,7 +349,7 @@ void PackageTool::showAppstreamInfo(const QString &pluginName)
     // with the argument we'll print to the defined path.
     // TODO: in KF6 we should switch to argument-only.
     QIODevice *outputDevice = cout->device();
-    QScopedPointer<QFile> outputFile;
+    std::unique_ptr<QFile> outputFile;
     const auto outputPath = d->parser->value(Options::appstreamOutput());
     if (!outputPath.isEmpty()) {
         auto outputUrl = QUrl::fromUserInput(outputPath);
@@ -464,12 +358,11 @@ void PackageTool::showAppstreamInfo(const QString &pluginName)
             *cerr << "Failed to open output file for writing.";
             exit(1);
         }
-        outputDevice = outputFile.data();
+        outputDevice = outputFile.get();
     }
 
     if (i.description().isEmpty()) {
-        *cerr << "Error: description missing, will result in broken appdata field as <summary/> is mandatory at "
-              << QFileInfo(i.metaDataFileName()).absoluteFilePath();
+        *cerr << "Error: description missing, will result in broken appdata field as <summary/> is mandatory at " << QFileInfo(i.fileName()).absoluteFilePath();
         std::exit(10);
     }
 
@@ -484,7 +377,7 @@ void PackageTool::showAppstreamInfo(const QString &pluginName)
         writer.writeTextElement(QStringLiteral("extends"), parentApp);
     }
 
-    const QJsonObject rootObject = i.rawData()[QStringLiteral("KPlugin")].toObject();
+    const QJsonObject rootObject = i.rawData().value(QStringLiteral("KPlugin")).toObject();
     translateKPluginToAppstream(QStringLiteral("name"), QStringLiteral("Name"), rootObject, writer, false);
     translateKPluginToAppstream(QStringLiteral("summary"), QStringLiteral("Description"), rootObject, writer, false);
     if (!i.website().isEmpty()) {
@@ -506,9 +399,12 @@ void PackageTool::showAppstreamInfo(const QString &pluginName)
         QStringList authorsText;
         authorsText.reserve(authors.size());
         for (const auto &author : authors) {
-            authorsText += QStringLiteral("%1 <%2>").arg(author.name(), author.emailAddress());
+            authorsText += QStringLiteral("%1").arg(author.name());
         }
-        writer.writeTextElement(QStringLiteral("developer_name"), authorsText.join(QStringLiteral(", ")));
+        writer.writeStartElement(QStringLiteral("developer"));
+        writer.writeAttribute(QStringLiteral("id"), QStringLiteral("kde.org"));
+        writer.writeTextElement(QStringLiteral("name"), authorsText.join(QStringLiteral(", ")));
+        writer.writeEndElement();
     }
 
     if (!i.iconName().isEmpty()) {
@@ -525,12 +421,10 @@ void PackageTool::showAppstreamInfo(const QString &pluginName)
     exit(0);
 }
 
-QString PackageTool::findPackageRoot(const QString &pluginName, const QString &prefix)
+QString PackageTool::resolvePackageRootWithOptions()
 {
-    Q_UNUSED(pluginName);
-    Q_UNUSED(prefix);
     QString packageRoot;
-    if (d->parser->isSet(Options::packageRoot()) && d->parser->isSet(Options::global()) && !d->parser->isSet(Options::generateIndex())) {
+    if (d->parser->isSet(Options::packageRoot()) && d->parser->isSet(Options::global())) {
         qWarning() << i18nc("The user entered conflicting options packageroot and global, this is the error message telling the user he can use only one",
                             "The packageroot and global options conflict with each other, please select only one.");
         ::exit(7);
@@ -548,9 +442,9 @@ QString PackageTool::findPackageRoot(const QString &pluginName, const QString &p
     return packageRoot;
 }
 
-void PackageTool::listPackages(const QStringList &types, const QString &path)
+void PackageTool::listPackages(const QString &kpackageType, const QString &path)
 {
-    QStringList list = d->packages(types, path);
+    QStringList list = d->packages(kpackageType, path);
     list.sort();
     for (const QString &package : std::as_const(list)) {
         d->coutput(package);
@@ -558,44 +452,35 @@ void PackageTool::listPackages(const QStringList &types, const QString &path)
     exit(0);
 }
 
-void PackageToolPrivate::renderTypeTable(const QMap<QString, QStringList> &plugins)
+void PackageToolPrivate::renderTypeTable(const QMap<QString, QString> &plugins)
 {
-    const QString nameHeader = i18n("Addon Name");
-    const QString pluginHeader = i18n("Service Type");
+    const QString nameHeader = i18n("KPackage Structure Name");
     const QString pathHeader = i18n("Path");
     int nameWidth = nameHeader.length();
-    int pluginWidth = pluginHeader.length();
     int pathWidth = pathHeader.length();
 
-    QMapIterator<QString, QStringList> pluginIt(plugins);
+    QMapIterator<QString, QString> pluginIt(plugins);
     while (pluginIt.hasNext()) {
         pluginIt.next();
         if (pluginIt.key().length() > nameWidth) {
             nameWidth = pluginIt.key().length();
         }
 
-        if (pluginIt.value()[0].length() > pluginWidth) {
-            pluginWidth = pluginIt.value()[0].length();
-        }
-
-        if (pluginIt.value()[1].length() > pathWidth) {
-            pathWidth = pluginIt.value()[1].length();
+        if (pluginIt.value().length() > pathWidth) {
+            pathWidth = pluginIt.value().length();
         }
     }
 
-    std::cout << nameHeader.toLocal8Bit().constData() << std::setw(nameWidth - nameHeader.length() + 2) << ' ' << pluginHeader.toLocal8Bit().constData()
-              << std::setw(pluginWidth - pluginHeader.length() + 2) << ' ' << pathHeader.toLocal8Bit().constData()
+    std::cout << nameHeader.toLocal8Bit().constData() << std::setw(nameWidth - nameHeader.length() + 2) << ' ' << pathHeader.toLocal8Bit().constData()
               << std::setw(pathWidth - pathHeader.length() + 2) << ' ' << std::endl;
-    std::cout << std::setfill('-') << std::setw(nameWidth) << '-' << "  " << std::setw(pluginWidth) << '-' << "  " << std::setw(pathWidth) << '-' << "  "
-              << std::endl;
+    std::cout << std::setfill('-') << std::setw(nameWidth) << '-' << "  " << std::setw(pathWidth) << '-' << "  " << std::endl;
     std::cout << std::setfill(' ');
 
     pluginIt.toFront();
     while (pluginIt.hasNext()) {
         pluginIt.next();
         std::cout << pluginIt.key().toLocal8Bit().constData() << std::setw(nameWidth - pluginIt.key().length() + 2) << ' '
-                  << pluginIt.value()[0].toLocal8Bit().constData() << std::setw(pluginWidth - pluginIt.value()[0].length() + 2) << ' '
-                  << pluginIt.value()[1].toLocal8Bit().constData() << std::setw(pathWidth - pluginIt.value()[1].length() + 2) << std::endl;
+                  << pluginIt.value().toLocal8Bit().constData() << std::setw(pathWidth - pluginIt.value().length() + 2) << std::endl;
     }
 }
 
@@ -604,42 +489,41 @@ void PackageToolPrivate::listTypes()
     coutput(i18n("Package types that are installable with this tool:"));
     coutput(i18n("Built in:"));
 
-    QMap<QString, QStringList> builtIns;
-    builtIns.insert(i18n("KPackage/Generic"),
-                    QStringList() << QStringLiteral("KPackage/Generic") << QStringLiteral(KPACKAGE_RELATIVE_DATA_INSTALL_DIR "/packages/"));
-    builtIns.insert(i18n("KPackage/GenericQML"),
-                    QStringList() << QStringLiteral("KPackage/GenericQML") << QStringLiteral(KPACKAGE_RELATIVE_DATA_INSTALL_DIR "/genericqml/"));
+    QMap<QString, QString> builtIns;
+    builtIns.insert(i18n("KPackage/Generic"), QStringLiteral(KPACKAGE_RELATIVE_DATA_INSTALL_DIR "/packages/"));
+    builtIns.insert(i18n("KPackage/GenericQML"), QStringLiteral(KPACKAGE_RELATIVE_DATA_INSTALL_DIR "/genericqml/"));
 
     renderTypeTable(builtIns);
 
-    const QVector<KPluginMetaData> offers = KPluginMetaData::findPlugins(QStringLiteral("kpackage/packagestructure"));
+    const QList<KPluginMetaData> offers = KPluginMetaData::findPlugins(QStringLiteral("kf6/packagestructure"));
 
     if (!offers.isEmpty()) {
         std::cout << std::endl;
         coutput(i18n("Provided by plugins:"));
 
-        QMap<QString, QStringList> plugins;
+        QMap<QString, QString> plugins;
         for (const KPluginMetaData &info : offers) {
-            KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage(info.pluginId());
-            QString name = info.name();
-            QString plugin = info.pluginId();
-            QString path = pkg.defaultPackageRoot();
-            plugins.insert(name, QStringList() << plugin << path);
+            const QString type = readKPackageType(info);
+            if (type.isEmpty()) {
+                continue;
+            }
+            KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage(type);
+            plugins.insert(type, pkg.defaultPackageRoot());
         }
 
         renderTypeTable(plugins);
     }
 }
 
-void PackageTool::packageInstalled(KJob *job)
+void PackageTool::packageInstalled(KPackage::PackageJob *job)
 {
     bool success = (job->error() == KJob::NoError);
     int exitcode = 0;
     if (success) {
         if (d->parser->isSet(Options::upgrade())) {
-            d->coutput(i18n("Successfully upgraded %1", d->packageFile));
+            d->coutput(i18n("Successfully upgraded %1", job->package().path()));
         } else {
-            d->coutput(i18n("Successfully installed %1", d->packageFile));
+            d->coutput(i18n("Successfully installed %1", job->package().path()));
         }
     } else {
         d->cerror(i18n("Error: Installation of %1 failed: %2", d->packageFile, job->errorText()));
@@ -648,20 +532,20 @@ void PackageTool::packageInstalled(KJob *job)
     exit(exitcode);
 }
 
-void PackageTool::packageUninstalled(KJob *job)
+void PackageTool::packageUninstalled(KPackage::PackageJob *job)
 {
     bool success = (job->error() == KJob::NoError);
     int exitcode = 0;
     if (success) {
         if (d->parser->isSet(Options::upgrade())) {
             d->coutput(i18n("Upgrading package from file: %1", d->packageFile));
-            KJob *installJob = d->installer.install(d->packageFile, d->packageRoot);
-            // clang-format off
-            connect(installJob, SIGNAL(result(KJob*)), SLOT(packageInstalled(KJob*)));
-            // clang-format on
+            auto installJob = KPackage::PackageJob::install(d->kpackageType, d->packageFile, d->packageRoot);
+            connect(installJob, &KPackage::PackageJob::finished, this, [installJob, this]() {
+                packageInstalled(installJob);
+            });
             return;
         }
-        d->coutput(i18n("Successfully uninstalled %1", d->packageFile));
+        d->coutput(i18n("Successfully uninstalled %1", job->package().path()));
     } else {
         d->cerror(i18n("Error: Uninstallation of %1 failed: %2", d->packageFile, job->errorText()));
         exitcode = 7;
